'use strict';

var jsxRuntime = require('react/jsx-runtime');
var Taro = require('@tarojs/taro');
var components = require('@tarojs/components');
var react = require('react');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Taro__default = /*#__PURE__*/_interopDefaultLegacy(Taro);

function isString(v) {
    return typeof v === 'string';
}
function isFunction(v) {
    return typeof v === 'function';
}
function isEqual(x, y) {
    if (x === y) {
        return true;
    }
    else if (typeof x === 'object' && x !== null && typeof y === 'object' && y !== null) {
        const keysX = Object.keys(x);
        const keysY = Object.keys(y);
        if (keysX.length !== keysY.length) {
            return false;
        }
        for (const key of keysX) {
            if (!isEqual(x[key], y[key])) {
                return false;
            }
        }
        return true;
    }
    else {
        return false;
    }
}
function pick(obj = {}, keys) {
    const r = {};
    keys.forEach((key) => {
        r[key] = obj[key];
    });
    return r;
}
const idCounter = {};
function uniqueId(prefix = '$unique$') {
    if (!idCounter[prefix]) {
        idCounter[prefix] = 0;
    }
    const id = ++idCounter[prefix];
    if (prefix === '$unique$') {
        return `${id}`;
    }
    return `${prefix}${id}`;
}
function compareVersion(v1, v2) {
    v1 = v1.split('.');
    v2 = v2.split('.');
    const len = Math.max(v1.length, v2.length);
    while (v1.length < len) {
        v1.push('0');
    }
    while (v2.length < len) {
        v2.push('0');
    }
    for (let i = 0; i < len; i++) {
        const num1 = parseInt(v1[i]);
        const num2 = parseInt(v2[i]);
        if (num1 > num2) {
            return 1;
        }
        else if (num1 < num2) {
            return -1;
        }
    }
    return 0;
}
function tripleDefer(func) {
    setTimeout(() => setTimeout(() => setTimeout(() => func())));
}

class WxCanvas {
    constructor(ctx, isNew, canvasNode) {
        this.ctx = ctx;
        this.chart = null;
        if (isNew) {
            this.canvasNode = canvasNode;
        }
        else {
            this._initStyle(ctx);
        }
        this._initEvent();
    }
    getContext(contextType) {
        if (contextType === '2d') {
            return this.ctx;
        }
    }
    setChart(chart) {
        this.chart = chart;
    }
    attachEvent() {
        // noop
    }
    addEventListener() {
        // noop
    }
    detachEvent() {
        // noop
    }
    _initCanvas(zrender, ctx) {
        zrender.util.getContext = function () {
            return ctx;
        };
        zrender.util.$override('measureText', function (text, font) {
            ctx.font = font || '12px sans-serif';
            return ctx.measureText(text);
        });
    }
    _initStyle(ctx) {
        var styles = [
            'fillStyle',
            'strokeStyle',
            'globalAlpha',
            'textAlign',
            'textBaseAlign',
            'shadow',
            'lineWidth',
            'lineCap',
            'lineJoin',
            'lineDash',
            'miterLimit',
            'fontSize',
        ];
        styles.forEach(style => {
            Object.defineProperty(ctx, style, {
                set: value => {
                    if ((style !== 'fillStyle' && style !== 'strokeStyle') ||
                        (value !== 'none' && value !== null)) {
                        ctx['set' + style.charAt(0).toUpperCase() + style.slice(1)](value);
                    }
                },
            });
        });
        ctx.createRadialGradient = function () {
            return ctx.createCircularGradient.apply(ctx, arguments);
        };
    }
    _initEvent() {
        this.event = {};
        const eventNames = [
            {
                wxName: 'touchStart',
                ecName: 'mousedown',
            },
            {
                wxName: 'touchMove',
                ecName: 'mousemove',
            },
            {
                wxName: 'touchEnd',
                ecName: 'mouseup',
            },
            {
                wxName: 'touchEnd',
                ecName: 'click',
            },
        ];
        eventNames.forEach(name => {
            this.event[name.wxName] = e => {
                const touch = e.touches[0];
                this.chart.getZr().handler.dispatch(name.ecName, {
                    zrX: name.wxName === 'tap' ? touch.clientX : touch.x,
                    zrY: name.wxName === 'tap' ? touch.clientY : touch.y,
                });
            };
        });
    }
    set width(w) {
        if (this.canvasNode)
            this.canvasNode.width = w;
    }
    set height(h) {
        if (this.canvasNode)
            this.canvasNode.height = h;
    }
    get width() {
        if (this.canvasNode)
            return this.canvasNode.width;
        return 0;
    }
    get height() {
        if (this.canvasNode)
            return this.canvasNode.height;
        return 0;
    }
}

function usePrevious(value) {
    const ref = react.useRef();
    react.useEffect(() => {
        ref.current = value;
    });
    return ref.current;
}

function useUnMount(fn) {
    react.useEffect(() => fn, []);
}

function useFirstMountState() {
    const isFirst = react.useRef(true);
    if (isFirst.current) {
        isFirst.current = false;
        return true;
    }
    return isFirst.current;
}

function useUpdateEffect(effect, deps) {
    const isFirstMount = useFirstMountState(); // 是否首次渲染
    react.useEffect(() => {
        if (!isFirstMount) {
            // 如果不是首次，则执行 effect 函数
            return effect();
        }
    }, deps);
}

function wrapTouch(event) {
    for (let i = 0; i < event.touches.length; ++i) {
        const touch = event.touches[i];
        touch.offsetX = touch.x;
        touch.offsetY = touch.y;
    }
    return event;
}
function touchStart({ chart, event }) {
    if (chart && event.touches.length > 0) {
        const touch = event.touches[0];
        const handler = chart.getZr().handler;
        handler.dispatch('mousedown', {
            zrX: touch.x,
            zrY: touch.y,
            preventDefault: () => { },
            stopImmediatePropagation: () => { },
            stopPropagation: () => { },
        });
        handler.dispatch('mousemove', {
            zrX: touch.x,
            zrY: touch.y,
            preventDefault: () => { },
            stopImmediatePropagation: () => { },
            stopPropagation: () => { },
        });
        handler.processGesture(wrapTouch(event), 'start');
    }
}
function touchMove({ chart, event }) {
    if (chart && event.touches.length > 0) {
        const touch = event.touches[0];
        const handler = chart.getZr().handler;
        handler.dispatch('mousedown', {
            zrX: touch.x,
            zrY: touch.y,
            preventDefault: () => { },
            stopImmediatePropagation: () => { },
            stopPropagation: () => { },
        });
        handler.dispatch('mousemove', {
            zrX: touch.x,
            zrY: touch.y,
            preventDefault: () => { },
            stopImmediatePropagation: () => { },
            stopPropagation: () => { },
        });
        handler.processGesture(wrapTouch(event), 'start');
    }
}
function touchEnd({ chart, event }) {
    if (chart) {
        const touch = event.changedTouches ? event.changedTouches[0] : {};
        const handler = chart.getZr().handler;
        handler.dispatch('mouseup', {
            zrX: touch.x,
            zrY: touch.y,
            preventDefault: () => { },
            stopImmediatePropagation: () => { },
            stopPropagation: () => { },
        });
        handler.dispatch('click', {
            zrX: touch.x,
            zrY: touch.y,
            preventDefault: () => { },
            stopImmediatePropagation: () => { },
            stopPropagation: () => { },
        });
        handler.processGesture(wrapTouch(event), 'end');
    }
}

const Echarts = ({ echarts, isPage = true, canvasId: pCanvasId, ...props }) => {
    const canvasRef = react.useRef();
    const chartRef = react.useRef();
    const [isInitialResize, setIsInitialResize] = react.useState(true);
    const prevProps = usePrevious(props);
    const canvasId = react.useMemo(() => pCanvasId || uniqueId('canvas_'), [pCanvasId]);
    const canvasProps = react.useMemo(() => [
        'disableScroll',
        'disableScroll',
        'onTouchCancel',
        'onLongTap',
        'onError',
        'nativeProps',
        'className',
        'key',
        'hidden',
        'animation',
    ], []);
    /**
     * issues: https://github.com/NervJS/taro/issues/7116
     * 获取小程序渲染层的节点要在 onReady 生命周期，等同于 useReady hooks
     * 访问小程序渲染层的 DOM 节点。
     */
    Taro.useReady(() => {
        // 顶层页面级别才触发useReady 【注意Popup 、Dialog 等弹出层 都不是页面级别】
        if (Taro__default["default"].getEnv() === Taro__default["default"].ENV_TYPE.WEAPP && isPage) {
            Taro.nextTick(() => {
                initChart();
            });
        }
    });
    react.useEffect(() => {
        if (Taro__default["default"].getEnv() === Taro__default["default"].ENV_TYPE.WEB || !isPage) {
            tripleDefer(() => {
                Taro.nextTick(() => {
                    initChart();
                });
            });
        }
    }, []);
    useUnMount(() => {
        dispose();
    });
    useUpdateEffect(() => {
        if (!isEqual(prevProps === null || prevProps === void 0 ? void 0 : prevProps.theme, props.theme) ||
            !isEqual(prevProps === null || prevProps === void 0 ? void 0 : prevProps.opts, props.opts) ||
            !isEqual(prevProps === null || prevProps === void 0 ? void 0 : prevProps.onEvents, props.onEvents)) {
            dispose();
            initChart(); // re-render
            return;
        }
        // update
        const pickKeys = ['option', 'notMerge', 'lazyUpdate', 'showLoading', 'loadingOption'];
        if (!isEqual(pick(props, pickKeys), pick(prevProps, pickKeys))) {
            updateEChartsOption();
        }
        /**
         * resize: style 、className
         */
        if (!isEqual(prevProps === null || prevProps === void 0 ? void 0 : prevProps.style, props.style) ||
            !isEqual(prevProps === null || prevProps === void 0 ? void 0 : prevProps.className, props.className)) {
            resize(canvasRef.current);
        }
    }, [props]);
    // 大小变化
    const resize = (canvas) => {
        const echartsInstance = echarts.getInstanceByDom(canvas);
        // 调整大小不应在第一次渲染时发生，因为它会取消初始 echarts 动画
        if (!isInitialResize) {
            try {
                echartsInstance.resize({
                    width: 'auto',
                    height: 'auto',
                });
            }
            catch (e) {
                console.warn(e);
            }
        }
        setIsInitialResize(false);
    };
    const initEchartsInstance = async ({ width, height, devicePixelRatio }) => {
        const { theme, opts } = props;
        return new Promise((resolve, reject) => {
            if (canvasRef.current) {
                chartRef.current = echarts.init(canvasRef.current, theme, {
                    width,
                    height,
                    devicePixelRatio,
                    ...opts,
                });
                if (Taro__default["default"].getEnv() === Taro__default["default"].ENV_TYPE.WEB) {
                    /**
                     * echart同一个dom下多次动态渲染值，防止值、事件重复互相影响
                     * 每次init之后，先dispose释放下资源，再重新init
                     */
                    const echartsInstance = echarts.getInstanceByDom(canvasRef.current);
                    echartsInstance.on('finished', () => {
                        echarts.dispose(canvasRef.current);
                        // 获取渲染后的width、height
                        const newOpts = {
                            width,
                            height,
                            devicePixelRatio,
                            ...opts,
                        };
                        chartRef.current = echarts.init(canvasRef.current, theme, newOpts);
                        resolve(chartRef.current);
                    });
                }
                else {
                    resolve(chartRef.current);
                }
            }
            else {
                reject(null);
            }
        });
    };
    const updateEChartsOption = () => {
        /**
         *  官方文档：https://echarts.apache.org/zh/api.html#echartsInstance.setOption
         */
        const { option, notMerge = true, // 不跟之前设置的option合并，保证每次渲染都是最新的option
        lazyUpdate = false, // 设置完 option 后是否不立即更新图表，默认为 false，即同步立即更新。如果为 true，则会在下一个 animation frame 中，才更新图表
        showLoading, loadingOption = null, } = props;
        // 1. 获取echarts实例
        const echartInstance = echarts.getInstanceByDom(canvasRef.current);
        if (echartInstance) {
            // 2. 设置option
            echartInstance.setOption(option, notMerge, lazyUpdate);
            // 3. 显示加载动画效果
            if (showLoading)
                echartInstance.showLoading(loadingOption);
            else
                echartInstance.hideLoading();
        }
        return echartInstance;
    };
    // 绑定事件
    const bindEvents = (instance, events) => {
        function _bindEvent(eventName, func) {
            if (isString(eventName) && isFunction(func)) {
                instance.on(eventName, (param) => {
                    func(param, instance);
                });
            }
        }
        for (const eventName in events) {
            if (Object.prototype.hasOwnProperty.call(events, eventName)) {
                _bindEvent(eventName, events[eventName]);
            }
        }
    };
    // 渲染图表
    const renderEcharts = async ({ width, height, devicePixelRatio }) => {
        const { onEvents, onChartReady } = props;
        // 1. 初始化图表
        await initEchartsInstance({
            width,
            height,
            devicePixelRatio,
        });
        // 2. 更新echarts实例
        const echartsInstance = updateEChartsOption();
        // 3. 绑定事件
        bindEvents(echartsInstance, onEvents || {});
        // 4. 图表渲染完成
        if (isFunction(onChartReady))
            onChartReady === null || onChartReady === void 0 ? void 0 : onChartReady(echartsInstance);
        // 5. resize
        if (canvasRef.current) {
            resize(canvasRef.current);
        }
    };
    // 销毁echarts实例
    const dispose = () => {
        if (canvasRef.current) {
            echarts === null || echarts === void 0 ? void 0 : echarts.dispose(canvasRef.current);
        }
    };
    // 初始化微信小程序图表
    const initWexinChart = () => {
        const query = Taro__default["default"].createSelectorQuery();
        query
            .select(`#${canvasId}`)
            .fields({
            node: true,
            size: true,
        })
            .exec((res) => {
            const [result] = res;
            if (result) {
                const { node, width, height } = result || {};
                const canvasNode = node;
                const canvasDpr = Taro__default["default"].getSystemInfoSync().pixelRatio;
                const ctx = canvasNode.getContext('2d');
                const canvas = new WxCanvas(ctx, true, canvasNode);
                echarts === null || echarts === void 0 ? void 0 : echarts.setCanvasCreator(() => {
                    return canvas;
                });
                canvasRef.current = canvas;
                renderEcharts({
                    width,
                    height,
                    devicePixelRatio: canvasDpr,
                });
            }
        });
    };
    // 初始化图表
    const initChart = () => {
        var _a, _b, _c, _d;
        if (Taro__default["default"].getEnv() === Taro__default["default"].ENV_TYPE.WEB && canvasRef.current) {
            const width = ((_a = props.style) === null || _a === void 0 ? void 0 : _a.width) || ((_b = canvasRef.current) === null || _b === void 0 ? void 0 : _b.clientWidth) || Taro__default["default"].getSystemInfoSync().windowWidth;
            const height = ((_c = props.style) === null || _c === void 0 ? void 0 : _c.height) || ((_d = canvasRef.current) === null || _d === void 0 ? void 0 : _d.clientHeight) || 300;
            renderEcharts({
                width,
                height,
                devicePixelRatio: window.devicePixelRatio,
            });
        }
        else {
            const version = Taro__default["default"].getSystemInfoSync().SDKVersion;
            const canUseNewCanvas = compareVersion(version, '2.9.0') >= 0;
            if (canUseNewCanvas) {
                // console.log('微信基础库版本大于2.9.0，开始使用<canvas type="2d"/>');
                // 2.9.0 可以使用 <canvas type="2d"></canvas>
                initWexinChart();
            }
            else {
                console.error(`当前基础库为${version}, 微信基础库版本过低，需大于等于 2.9.0`);
            }
        }
    };
    // container component
    const renderContainerComponent = react.useMemo(() => {
        if (Taro__default["default"].getEnv() === Taro__default["default"].ENV_TYPE.WEAPP) {
            return (jsxRuntime.jsx(components.Canvas, { type: '2d', id: canvasId, canvasId: canvasId, style: {
                    width: '100%',
                    height: '300px',
                    ...props.style,
                }, ref: canvasRef, onTouchStart: (event) => touchStart({ chart: chartRef.current, event }), onTouchMove: (event) => touchMove({ chart: chartRef.current, event }), onTouchEnd: (event) => touchEnd({ chart: chartRef.current, event }), ...pick(props, canvasProps) }));
        }
        return (jsxRuntime.jsx(components.View, { ref: canvasRef, id: canvasId, style: {
                width: '100%',
                height: '300px',
                ...props.style,
            } }));
    }, [props, canvasProps, canvasId]);
    return renderContainerComponent;
};
var Echarts$1 = react.memo(Echarts);

module.exports = Echarts$1;
//# sourceMappingURL=index.js.map
